package com.atlassian.confluence.jspwiki;

import com.atlassian.confluence.setup.ConfluenceSetup;
import com.ecyrd.jspwiki.PageManager;
import com.ecyrd.jspwiki.attachment.AttachmentManager;
import net.sf.hibernate.SessionFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.FileNotFoundException;
import java.util.Properties;

public class ConfluenceExporter
{
    static AttachmentManager attachmentManager;
    static PageManager pageManager;
    static String spaceName;
    static String creatorUserName;
    static Properties properties = new Properties();

    public static void main(String[] args) throws Exception
    {
        if (args.length < 3 || args.length > 3)
        {
            System.out.println("syntax: java -jar jspwiki-exporter-0.1.jar path/to/jspwiki.properties spaceName creatorUserName");
            return;
        }

        File jspwikiPropertiesFile = null;
        File file = new File(args[0]);
        if (file.isFile())
        {
            jspwikiPropertiesFile = file;
        }
        else if (file.isDirectory())
        {
            jspwikiPropertiesFile = new File(file, "jspwiki.properties");
        }

        spaceName = args[1];
        creatorUserName = args[2];

        if (jspwikiPropertiesFile == null)
            throw new FileNotFoundException("Error accessing jspwiki.properties: Please check the path to this file is correct");

        export(jspwikiPropertiesFile);

        System.out.println("JSPWiki export complete.");
        System.exit(0);
    }

    public static String export(File jspwikiPropertiesFile) throws Exception
    {
        instantiateManagers(jspwikiPropertiesFile);
        return exportEverything();
    }

    static void instantiateManagers(File jspwikiPropertiesFile) throws Exception
    {
        InputStream propertyStream = null;
        propertyStream = new FileInputStream(jspwikiPropertiesFile);

        properties.load(propertyStream);

        pageManager       = new PageManager(properties);
        attachmentManager = new AttachmentManager(null, properties);
    }

    static String exportEverything() throws Exception
    {
        ConfluenceSetup.getInstance().setProperty(ConfluenceSetup.TEMP_DIR_PROP, ".");
        JspWikiBackupExporter jspWikiBackupExporter = new JspWikiBackupExporter(pageManager, attachmentManager, spaceName, creatorUserName, properties);

        ClassPathXmlApplicationContext appContext = new ClassPathXmlApplicationContext("/applicationContext.xml");
        SessionFactory sessionFactory = (SessionFactory)appContext.getBean("sessionFactory");

        jspWikiBackupExporter.setSessionFactory(sessionFactory);
        String zipFilePath = jspWikiBackupExporter.doExport();
        return zipFilePath;
    }

}
