package com.atlassian.confluence.jspwiki;

import junit.framework.TestCase;

import java.util.Properties;
import java.io.*;

import com.atlassian.confluence.importexport.impl.snipsnap.RegExpProcessor;

public class TestContentCleaner extends TestCase
{
    ContentCleaner cleaner;

    protected void setUp() throws Exception
    {
        super.setUp();

        Properties variables = new Properties();
        variables.setProperty(ContentCleaner.VAR_APPLICATION_NAME, "My application");
        variables.setProperty(ContentCleaner.VAR_ENCODING, "UTF-666");
        variables.setProperty(ContentCleaner.VAR_JSPWIKI_VERSION, "6.6.6");
        variables.setProperty(ContentCleaner.VAR_PAGE_NAME, "A Page");
        variables.setProperty(ContentCleaner.VAR_TOTAL_PAGES, "666");
        cleaner = new ContentCleaner(variables);
    }

    public void testContentCleaner() throws Exception
    {
        String source = "here we go:\n" +
                "----\n" +
                "\\\\\n" +
                "[link]     = creates a hyperlink to an internal WikiPage called 'Link'.\n" +
                "[click here|link] = creates a hyperlink to an internal WikiPage called\n" +
                "'Link', but displays the text 'click here' to the\n" +
                "user instead of 'Link'.\n" +
                "[1]        = Makes a reference to a footnote numbered 1.\n" +
                "[#1]       = Marks the footnote number 1.\n" +
                "\n" +
                "!small heading\n" +
                "!!medium heading\n" +
                "!!!large heading\n" +
                "\n" +
                "''text''   = prints 'text' in italic.\n" +
                "__text__   = prints 'text' in bold.\n" +
                "{{text}}   = prints 'text' in monospaced font.\n" +
                "\n" +
                "* text     = makes a bulleted list item with 'text'\n" +
                "** text    = makes a bulleted list item with 'text'\n" +
                "# text     = makes a numbered list item with 'text'\n" +
                "## text    = makes a numbered list item with 'text'\n" +
                ";term:ex   = makes a definition for 'term' with the explanation 'ex'\n" +
                "an image here: [http://example.com/example.png]\n" +
                "an attachment: [bla.txt]\n" +
                "another attachment: [page/bla.txt]\n" +
                "a png attachment: [bla.png]\n" +
                "but this not an attachment: [http://java.sun.com]\n" +
                "|| Heading 1 || Heading 2\n" +
                "| ''Gobble'' | Bar\n" +
                "| [Main]     | [SandBox]\n" +
                "[{$applicationname}]\n" +
                "[{$encoding}]\n" +
                "[{$jspwikiversion}]\n" +
                "[{$pagename}]\n" +
                "[{$totalpages}]\n" +
                "{{{__blabla__ ''blabla''}}}" +
                "Lets have a look at whether the ''jspwiki exporter'' drops space characters when converting markup characters" +
                "what about a variable like $foobar" +
                "[http://www.google.com/logo.gif]" +
                "[http://www.google.com/logo.jpg]" +
                "[http://www.google.com/logo.png]";
        String expected = "here we go:\n" +
                "----\n" +
                "\\\\\n" +
                "[link]     = creates a hyperlink to an internal WikiPage called 'Link'.\n" +
                "[click here|link] = creates a hyperlink to an internal WikiPage called\n" +
                "'Link', but displays the text 'click here' to the\n" +
                "user instead of 'Link'.\n" +
                "[1]        = Makes a reference to a footnote numbered 1.\n" +
                "[#1]       = Marks the footnote number 1.\n" +
                "\n" +
                "h4. small heading\n" +
                "h3. medium heading\n" +
                "h2. large heading\n" +
                "\n" +
                "_text_   = prints 'text' in italic.\n" +
                "*text*   = prints 'text' in bold.\n" +
                "{{text}}   = prints 'text' in monospaced font.\n" +
                "\n" +
                "* text     = makes a bulleted list item with 'text'\n" +
                "** text    = makes a bulleted list item with 'text'\n" +
                "# text     = makes a numbered list item with 'text'\n" +
                "## text    = makes a numbered list item with 'text'\n" +
                ";term:ex   = makes a definition for 'term' with the explanation 'ex'\n" +
                "an image here: !http://example.com/example.png!\n" +
                "an attachment: [^bla.txt]\n" +
                "another attachment: [page^bla.txt]\n" +
                "a png attachment: !bla.png!\n" +
                "but this not an attachment: [http://java.sun.com]\n" +
                "|| Heading 1 || Heading 2\n" +
                "| _Gobble_ | Bar\n" +
                "| [Main]     | [SandBox]\n" +
                "My application\n" +
                "UTF-666\n" +
                "6.6.6\n" +
                "A Page\n" +
                "666\n" +
                "{noformat}__blabla__ ''blabla''{noformat}" +
                "Lets have a look at whether the _jspwiki exporter_ drops space characters when converting markup characters" +
                "what about a variable like $foobar" +
                "!http://www.google.com/logo.gif!" +
                "!http://www.google.com/logo.jpg!" +
                "!http://www.google.com/logo.png!";
        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }

    // content cleaner should not format anything between the {{{ and }}}
    public void testCleanJSPWikiFormattingRulesPage()
    {
        String source = "When you've figured out how the editor works, then you should read [WikiEtiquette] so that you would know how to use your newly acquired skills...\n" +
                "\n" +
                "!Quick reference\n" +
                "\n" +
                "{{{\n" +
                "----       = make a horizontal ruler. Extra '-' is ignored.\n" +
                "\\\\         = force a line break\n" +
                "\n" +
                "[link]     = create a hyperlink to an internal WikiPage called 'Link'.\n" +
                "[this is also a link] = create a hyperlink to an internal WikiPage called \n" +
                "                        'ThisIsAlsoALink' but show the link as typed with spaces.\n" +
                "[a sample|link] = create a hyperlink to an internal WikiPage called\n" +
                "                    'Link', but display the text 'a sample' to the\n" +
                "                    user instead of 'Link'.\n" +
                "~NoLink    = disable link creation for the word in CamelCase.\n" +
                "[1]        = make a reference to a footnote numbered 1.\n" +
                "[#1]       = mark the footnote number 1.\n" +
                "[[link]    = create text '[link]'.\n" +
                "\n" +
                "!heading   = small heading with text 'heading'\n" +
                "!!heading  = medium heading with text 'heading'\n" +
                "!!!heading = large heading with text 'heading'\n" +
                "\n" +
                "''text''   = print 'text' in italic.\n" +
                "__text__   = print 'text' in bold.\n" +
                "{{text}}   = print 'text' in monospaced font.\n" +
                "\n" +
                "* text     = make a bulleted list item with 'text'\n" +
                "# text     = make a numbered list item with 'text'\n" +
                ";term:ex   = make a definition for 'term' with the explanation 'ex'\n" +
                "$jspwiki   = jspwikivariable\n" +
                "}}}\n" +
                "[Confluence|http://www.atlassian.org] and with a normal url link: [http://www.google.com]\n" +
                "Hello World. Welcome!\n" +
                "[Link|TestDocs/map.xls]\n" +
                "[link alias|the first page]\n" +
                "[file:\\\\ZZSERVER\\DATA\\Directory One\\Directory Two\\My Test.doc]\n" +
                "some list items\n" +
                "*item1\n" +
                "*item2\n" +
                "   *item3\n" +
                "*some normal text * that contains some asterisks *that should not be affected when trying to space out bullets and their matching text\n" +
                "[file link|file://server/main dir/dev dir/test file.txt]\n" +
                "[^File names with spaces should stay the same.doc]";
        String expected = "When you've figured out how the editor works, then you should read [WikiEtiquette] so that you would know how to use your newly acquired skills...\n" +
                "\n" +
                "h4. Quick reference\n" +
                "\n" +
                "{noformat}\n" +
                "----       = make a horizontal ruler. Extra '-' is ignored.\n" +
                "\\\\         = force a line break\n" +
                "\n" +
                "[link]     = create a hyperlink to an internal WikiPage called 'Link'.\n" +
                "[this is also a link] = create a hyperlink to an internal WikiPage called \n" +
                "                        'ThisIsAlsoALink' but show the link as typed with spaces.\n" +
                "[a sample|link] = create a hyperlink to an internal WikiPage called\n" +
                "                    'Link', but display the text 'a sample' to the\n" +
                "                    user instead of 'Link'.\n" +
                "~NoLink    = disable link creation for the word in CamelCase.\n" +
                "[1]        = make a reference to a footnote numbered 1.\n" +
                "[#1]       = mark the footnote number 1.\n" +
                "[[link]    = create text '[link]'.\n" +
                "\n" +
                "!heading   = small heading with text 'heading'\n" +
                "!!heading  = medium heading with text 'heading'\n" +
                "!!!heading = large heading with text 'heading'\n" +
                "\n" +
                "''text''   = print 'text' in italic.\n" +
                "__text__   = print 'text' in bold.\n" +
                "{{text}}   = print 'text' in monospaced font.\n" +
                "\n" +
                "* text     = make a bulleted list item with 'text'\n" +
                "# text     = make a numbered list item with 'text'\n" +
                ";term:ex   = make a definition for 'term' with the explanation 'ex'\n" +
                "$jspwiki   = jspwikivariable\n" +
                "{noformat}\n" +
                "[Confluence|http://www.atlassian.org] and with a normal url link: [http://www.google.com]\n" +
                "Hello World. Welcome!\n" +
                "[Link|TestDocs^map.xls]\n" +
                "[link alias|thefirstpage]\n" +
                "[file:\\\\ZZSERVER\\DATA\\Directory One\\Directory Two\\My Test.doc]\n" +
                "some list items\n" +
                "* item1\n" +
                "* item2\n" +
                "   * item3\n" +
                "*some normal text * that contains some asterisks *that should not be affected when trying to space out bullets and their matching text\n" +
                "[file link|file://server/main dir/dev dir/test file.txt]\n" +
                "[^File names with spaces should stay the same.doc]";
        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }

    public void testKnutCase1()
    {
        String source = "CoolCat provides no less than 3 different ways to do a ''copy presentation'':\n" +
                "* A keyboard shortcut: {{Ctrl-B}}\n" +
                "* A ''Copy Presentation'' in the context menu\n" +
                "* ''Operations > Copy Presentation'' ";

        String expected = "CoolCat provides no less than 3 different ways to do a _copy presentation_:\n" +
                "* A keyboard shortcut: {{Ctrl-B}}\n" +
                "* A _Copy Presentation_ in the context menu\n" +
                "* _Operations > Copy Presentation_ ";

        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }

    public void testKnutCase2()
    {
        String source = "* __XXX__: xxx";
        String expected = "* *XXX*: xxx";

        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }

    public void testKuntCase3()
    {
        String source = "[Icons/script.gif] __Die Form des fachlichen Inputs ist fr den Modellierer nicht relevant__, aber es gilt:";
        String expected = "!Icons/script.gif! *Die Form des fachlichen Inputs ist fr den Modellierer nicht relevant*, aber es gilt:";

        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }

    public void testKnutCase4()
    {
        String source = "# File > Page Setup\n" +
                "## __Paper Size__ = ISO A0 - 841 x 1169 mm\\\\__Scaling__ = Fit to 1 page(s) wide by 1 tall\\\\__Orientation__ = Landscape";

        String expected = "# File > Page Setup\n" +
                "## *Paper Size* = ISO A0 - 841 x 1169 mm\\\\*Scaling* = Fit to 1 page(s) wide by 1 tall\\\\*Orientation* = Landscape";

        String result = cleaner.clean(source);
        assertEquals(expected, result);

        /*RegExpProcessor r = new RegExpProcessor("(?<=[\\p{Space}])((?:[\\p{Upper}]{1}?[\\p{Lower}]+)++)|^((?:[\\p{Upper}]{1}?[\\p{Lower}]+)++)", "[$1$2]");
        System.out.println("r.process(\"HelloWorld sdfdsf sfdsda fsadf\") = " + r.process("HelloWorld went to the ShoppingCenter and boughtMilkCookies ATart"));*/
    }

    public void testHypenatedLinks()
    {
        String source = "[hello-to-world]\n" +
                "[test link|hello-world]";
        String expected = "[HelloToWorld]\n" +
                "[test link|HelloWorld]";

        String result = cleaner.clean(source);
        assertEquals(expected, result);
    }
}
